﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace CheckersGame
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
		//Global Variables
        List<System.Drawing.Point> PosMoves = new List<System.Drawing.Point>();
        System.Drawing.Point Current = new System.Drawing.Point();
        bool Turn;
        bool MultiTurn;
        claChecker ToBeRemoved;
        Rectangle[,] ds = new Rectangle[8, 8];

        public MainWindow()
        {
            //Game Is Opening
            InitializeComponent();

            //Start Game
            SetupGame();
        }

        private void SetupGame()
        {
            //Reset Objects
            claCheckers.Checkers.Clear();
            claTiles.Tiles.Clear();

            //Reset Turn
            Turn = false;

            //Load All Tile Info, Generate Objects
            #region TileLoad

			//From Setup File
            string[] bits = System.IO.File.ReadAllLines(@"Setup.txt");

			//Foreach Line Drawn From File
            foreach (var Tile in bits)
            {
				//Split It
                string[] parts = Tile.Split('@');

				//Initialize New Tile Object
                claTile T = new claTile();

				//Fill Data Appropriately From File
                T.Cord = new System.Drawing.Point(Convert.ToInt16(parts[0]), Convert.ToInt16(parts[1]));
                T.Checker = Convert.ToBoolean(parts[2]);
                T.Color = Convert.ToBoolean(parts[3]);

				//Add To Tile List
                claTiles.Tiles.Add(T);
            }

			//Foreach Tile
            foreach (var Tile in claTiles.Tiles)
            {
				//If Tile Colour Is True
                if (Tile.Color == true)
                {
					//Set To Black
                    Color(Tile.Cord.X, Tile.Cord.Y, Colors.Black);
                }
                else
                {
					//Otherwise Set To White
                    Color(Tile.Cord.X, Tile.Cord.Y, Colors.White);
                }
            }

            #endregion

            //Load All Checker Info, Generate Objects
            #region CheckerLoad

			//Load Infomation From CheckersSetup File
            string[] Pieces = System.IO.File.ReadAllLines(@"CheckerSetup.txt");

			//Foreach Line Drawn Out Of File
            foreach (var Piece in Pieces)
            {
				//Split The Line
                string[] parts = Piece.Split('@');

				//Initialize A New Checker Object
                claChecker Checker = new claChecker();

				//Fill In Details Of Checker Appropriately
                Checker.Cord = new System.Drawing.Point(Convert.ToInt16(parts[0]), Convert.ToInt16(parts[1]));
                Checker.Color = Convert.ToBoolean(parts[2]);
                Checker.King = false;
                Checker.Direction = Convert.ToInt16(parts[3]);

				//Add To List Of Checker Objects
                claCheckers.Checkers.Add(Checker);
            }

			//Foreach Checker
            foreach (var Checker in claCheckers.Checkers)
            {
				//Foreach Tile
                foreach (var Tile in claTiles.Tiles)
                {
					//If The Tile Matches The Checkers Co-ordinates
                    if (Tile.Cord == Checker.Cord)
						//Set That Rectangle To Show The Image Of A Checker With Appropriate Colour
                        Image(Checker.Cord.X, Checker.Cord.Y, Checker.Color);
                }
            }

            #endregion

            //Display Everything.
            DrawGrid();
        }

        private void Color(int X, int Y, Color color)
        {
            //Set Colour For The Important Tile.
            SolidColorBrush temp = new SolidColorBrush(color);

            ds[X, Y] = new Rectangle { Stroke = Brushes.Black, StrokeThickness = 2 };
            ds[X, Y].Fill = temp;
        }

        private void Image(int X, int Y, bool PColor)
        {
            //Choose Correct Image Based On Imported Pathing
            ds[X, Y] = new Rectangle { Stroke = Brushes.Black, StrokeThickness = 2 };
            ds[X, Y].Fill = new ImageBrush(new BitmapImage(new Uri(@"Checkers/" + PColor + ".jpg", UriKind.Relative)));
        }

        private void ShowNormMove(int x, int y, int Dir, bool Check)
        {
            //Returns This To The Default Of False
            MultiTurn = false;

            //Goes Through Every Checker
            foreach (var Checker in claCheckers.Checkers)
            {
                //If The Checker Matches The Place Clicked
                if (Checker.Cord.X == x & Checker.Cord.Y == y)
                {
                    //Checks Checker Clicked Is The Correct Checker Color.
                    if (Checker.Color != Turn)
                    {
                        //Sets Default Location Of Checker Selected.
                        Current = new System.Drawing.Point(x, y);

                        //Runs Through Every Tile.
                        foreach (var Tile in claTiles.Tiles)
                        {
                            //If The Tile Matches The Cords For 1 To The Left And 1 In Front. (TACKBACK#1)
                            if (Checker.Cord.X - 1 == Tile.Cord.X & Checker.Cord.Y - (1 * Dir) == Tile.Cord.Y)
                            {
                                //Checks Whether There Is A Checker On The Tile.
                                if (Tile.Checker == true)
                                {
                                    //If There Is A Checker On The Tile.

                                    //Creates New Bool
                                    bool Color = new bool();

                                    //This Stores Whether The Piece In Front Of The Checker
                                    //Is Opposite Or The Same.
                                    
                                    foreach (var oChecker in claCheckers.Checkers)
                                    {
                                        //If The Checker On The Tile We Found Before (TAGBACK#1)
                                        if (Tile.Cord.X == oChecker.Cord.X & Tile.Cord.Y == oChecker.Cord.Y)
                                        {
                                            //Set The Boolean To The Color Of That Checker.
                                            Color = oChecker.Color;
                                        }
                                    }

                                    //This Is Then Used To Check.
                                    if (Color == Turn)
                                    {
                                        //Runs Through Every Tile Again.
                                        foreach (var oTile in claTiles.Tiles)
                                        {
                                            //Finds The Tile 2 Ahead Of The Default Checker Position
                                            if (Checker.Cord.X - 2 == oTile.Cord.X & Checker.Cord.Y - (2 * Dir) == oTile.Cord.Y)
                                            {
                                                //If There Is Not A Checker On This Spot
                                                if (oTile.Checker == false)
                                                {
                                                    //Display Clear Spot As A Possible Movement
                                                    PosMoves.Add(oTile.Cord);
                                                    //If Checking For MultiTurn
                                                    if (Check == true)
                                                    {
                                                        //Set MultiTurn As True
                                                        MultiTurn = true;
                                                    }
                                                }
                                                //Else
                                                //There Is No Where For The Checker To Move To On This Side.
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    //If We Are Not Checking For MultiTurn
                                    if (Check == false)
                                    {
                                        //Display The Spot That Is Clear
                                        PosMoves.Add(Tile.Cord);
                                    }
                                }
                            }


                            //If We Find The Tile 1 Place Ahead To The Right Of The Default Checker Location
                            if (Checker.Cord.X + 1 == Tile.Cord.X & Checker.Cord.Y - (1 * Dir) == Tile.Cord.Y)
                            {
                                //If There Is A Checker On This Tile
                                if (Tile.Checker == true)
                                {
                                    //Creates New Bool
                                    bool Color = new bool();

                                    //This Stores Whether The Piece In Front Of The Checker
                                    //Is Opposite Or The Same.
                                    
                                    //Runs Through Every Checker
                                    foreach (var oChecker in claCheckers.Checkers)
                                    {
                                        //If The Checker Matches The Tile We Originally Found With The Checker On It
                                        if (Tile.Cord.X == oChecker.Cord.X & Tile.Cord.Y == oChecker.Cord.Y)
                                        {
                                            //Take The Color Of This Tile.
                                            Color = oChecker.Color;
                                        }
                                    }

                                    //This Is Then Used To Check.
                                    if (Color == Turn)
                                    {
                                        //Again Runs Through Every Tile.
                                        foreach (var oTile in claTiles.Tiles)
                                        {
                                            //Until We Find The Tile 2 Places In Front And To 
                                            //The Right Of The Default Checker Position
                                            if (Checker.Cord.X + 2 == oTile.Cord.X & Checker.Cord.Y - (2 * Dir) == oTile.Cord.Y)
                                            {
                                                //If There Is Not A Checker On This Position
                                                if (oTile.Checker == false)
                                                {
                                                    //Display Clear Spot As A Possible Movement
                                                    PosMoves.Add(oTile.Cord);
                                                    //If Checking For A MultiTurn
                                                    if (Check == true)
                                                    {
                                                        //Set MultiTurn To True
                                                        MultiTurn = true;
                                                    }
                                                }
                                                //Else
                                                //Checker Has No Where To Go To The Right Hand Side.
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    //If Not Checking For MultiTurn
                                    //(This Stops The Program From Allowing The Player)
                                    //(From Moving One Space After They Have Just Taken A Checker)
                                    if (Check == false)
                                    {
                                        //Display The Spot That Is Clear
                                        PosMoves.Add(Tile.Cord);

                                    }
                                }
                            }
                        }                        
                    }
                }
            }
        }

        private void ShowKingMove(int x, int y, bool Check)
        {
            //Clears All Possible Moves Before Finding New Ones
            PosMoves.Clear();

            //Does Normal Check Of Moves, In Both Directions
            ShowNormMove(x, y, 1, Check);
            ShowNormMove(x, y, -1, Check);
        }

        private void ResetGrid()
        {
            //Runs Through Every Tile.
            foreach (var Tile in claTiles.Tiles)
            {
                //If The Tile Color Is Black
                if (Tile.Color == true)
                {
                    //Color The Tile Black
                    Color(Tile.Cord.X, Tile.Cord.Y, Colors.Black);
                }
                else
                {
                    //Color The Tile White
                    Color(Tile.Cord.X, Tile.Cord.Y, Colors.White);
                }
            }

            //Go Through Every Checker.
            foreach (var Checker in claCheckers.Checkers)
            {
                //Display Image Of Correct Checker.
                Image(Checker.Cord.X, Checker.Cord.Y, Checker.Color);
            }
        }

        private void DrawGrid()
        {
            //Clear All Data (Tiles) Currently In The Child List
            canMain.Children.Clear();

            //8 x 8 Grid. Draw Out Each Square From The Rectangle Array.
            for (int i = 0; i < 8; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    //Set Margin For Each Square To Determine Where To Place It.
                    Canvas.SetLeft(ds[i, j], i * 50);
                    Canvas.SetTop(ds[i, j], j * 50);

                    //Set Width And Height Of Each Tile.
                    ds[i, j].Width = 50;
                    ds[i, j].Height = 50;

                    //Add Each To Child List Of Canvas.
                    canMain.Children.Add(ds[i, j]);
                }
            }
        }

        private void ChangeTurn()
        {
            //Inverts The Turns
            if (Turn == false)
            {
                Turn = true;
            }
            else
            {
                Turn = false;
            }
        }

        private void Restart(string Color)
        {
            //Calls In Which Color Has Won
            MessageBox.Show(Color + " Has Won");

            //Generic Restart Option
            if (MessageBox.Show("Would You Like To Play Again?", "Play Again?", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
            {
                //Starts Game Again
                SetupGame();
            }
            else
            {
                //Closes Application
                this.Close();
            }
        }

        private void CheckWin()
        {
            //Reset Counter For Black And White Checkers.
            int Red = 0;
            int Wheat = 0;

            //Run Through Every Checker
            foreach (var Checker in claCheckers.Checkers)
            {
                //If The Checker Is Red
                if (Checker.Color == false)
                {
                    //Add One To Red
                    Red++;
                }
                else
                {
                    //Add One To Wheat
                    Wheat++;
                }
            }

            //If There Are No Reds
            if (Red == 0)
            {
                //Wheat Has Won
                Restart("Wheat");
            }
            //If There Are No Wheats
            if (Wheat == 0)
            {
                //Red Has Won
                Restart("Red");
            }
        }

        private void canMain_MouseDown(object sender, MouseButtonEventArgs e)
        {
            //Identify X And Y Positions.
            #region Collecting Mouse Cords
            //get mouse cords
            Point pt = new Point();
            pt = Mouse.GetPosition(canMain);

            //Round To The Nearest Value And Assign Co-ords
            int X = (int)Math.Round(pt.X / 50.0);
            int Y = (int)Math.Round(pt.Y / 50.0);

            //Correction - The Clicking Was Off Center
            //This Means That If The Cord Point Is Bigger Than The Point It's Been Changed To
            //Take One Off

            //It basically seperates into the 4 portions of the square
            //This then corrects it based on which side it is on.
            if (pt.X - (X * 50) < 0)
            {
                X--;
            }
            if (pt.Y - (Y * 50) < 0)
            {
                Y--;
            }

            #endregion

            //If A Checker Is Moving
            #region Moving
            //If There Are Possible Moves Being Displayed
            //And If The User Clicks On One Of The Yellow Tiles
            if (PosMoves.Contains(new System.Drawing.Point(X, Y)))
            {
                //Overwrites Checker
                //Foreach Checker On The Board
                foreach (var Checker in claCheckers.Checkers)
                {
                    //If Current Checker, Matches The Checker In The List
                    if (Current.X == Checker.Cord.X & Checker.Cord.Y == Current.Y)
                    {
                        //Replace The Checkers X And Y With New Location
                        Checker.Cord = new System.Drawing.Point(X, Y);

                        if (Y == 0 | Y == 7)
                        {
                            Checker.King = true;
                        }
                    }
                }

                //Overwrites Tile Data To Change Where Tile.Checker Is Located.
                //Foreach Tile
                foreach (var Tile in claTiles.Tiles)
                {
                    //If Tile Matches The Previously Clicked Tiles Cords
                    if (Tile.Cord.X == Current.X & Tile.Cord.Y == Current.Y)
                    {
                        //Set That Tiles To Not Have A Checker
                        Tile.Checker = false;
                    }

                    //If Tile Matches Currently Clicked Cords
                    if (Tile.Cord.X == X & Tile.Cord.Y == Y)
                    {
                        //Set Tile To Have Checker.
                        Tile.Checker = true;
                    }
                }

                //Reset And Display Board.
                ResetGrid();
                PosMoves.Clear();

                //Checks For The Possibility Of Another Turn
                //Foreach Checker Currently On The Board
                foreach (var oChecker in claCheckers.Checkers)
                {
                    //If The Checker Matches The Currently Clicked Cords.
                    if (oChecker.Cord.X == X & oChecker.Cord.Y == Y)
                    {
                        //If A Checker Was Taken
                        if ((oChecker.Cord.X - 2 == Current.X | oChecker.Cord.X + 2 == Current.X) & (oChecker.Cord.Y - 2 == Current.Y | oChecker.Cord.Y + 2 == Current.Y))
                        {
                            //Determine Which Direction The Taking Piece Moved.
                            int tempX = ((oChecker.Cord.X - Current.X) / 2);
                            int tempY = ((oChecker.Cord.Y - Current.Y) / 2);

                            //Delete Taken Piece

                            //Go Through Every Checker
                            foreach (var Checker in claCheckers.Checkers)
                            {
                                //If The Checker Is The One That Has Just Been Jumped Over
                                if (Checker.Cord.X == (oChecker.Cord.X - tempX) & Checker.Cord.Y == (oChecker.Cord.Y - tempY))
                                {
                                    //Add Checker To Be Removed
                                    //We Have To Finish Going Through The List First.
                                    ToBeRemoved = Checker;

                                    //Goes Through Every Tile
                                    foreach (var Tile in claTiles.Tiles)
                                    {
                                        //Find The Tile That Has Been Jumped Over
                                        if (Tile.Cord.X == (oChecker.Cord.X - tempX) & Tile.Cord.Y == (oChecker.Cord.Y - tempY))
                                        {
                                            //Set Tile To Not Contain A Checker.
                                            Tile.Checker = false;
                                        }
                                    }
                                }
                            }

                            if (oChecker.King == true)
                            {
                                ShowKingMove(X, Y, true);
                            }
                            else
                            {
                                //Continue As If Normal Move To Check For Multiple Moves.
                                ShowNormMove(X, Y, oChecker.Direction, true);
                            }

                            //If There Are Possible Moves.
                            if (PosMoves.Count <= 0)
                            {
                                //Change Turn
                                ChangeTurn();
                            }
                        }
                        else
                        {
                            //Change Turn
                            ChangeTurn();
                        }
                    }
                }
            }
            #endregion

            //If A User Has Clicked A Checker
            #region ClickingChecker
            //If Not Clicking On A Spot Which Is Yellow
            else
            {
                //Provided We Are Not Waiting For The Player To MultiJump
                if (MultiTurn == false)
                {
                    //Check Through Every Checker
                    foreach (var Checker in claCheckers.Checkers)
                    {
                        //Find Correct Checker
                        if (X == Checker.Cord.X & Y == Checker.Cord.Y)
                        {
                            //Clear All Possible Moves.
                            //Before Adding New Ones.
                            PosMoves.Clear();

                            //If King, Move Like A King
                            if (Checker.King == true)
                            {
                                ShowKingMove(X, Y, false);
                                //Passes In X And Y Of Mouse Click.
                            }
                            //Else.. Move Normally.
                            else
                            {
                                ShowNormMove(X, Y, Checker.Direction, false);
                                //Passes In
                                //X And Y From Mouse Click
                                //Checker Direction, 1 Or -1 To Decide Which Way The Checker Is Going
                                //False, This Says We Are Not Checking To See Whether There - 
                                //Is A Possibility For A MultiJump Yet.
                            }
                        }
                    }
                }
            }
            #endregion

            //Everything That Needs To Be Done
            //At The End Of Every Turn
            #region EndingTurn
            //If We Have A Checker To Remove, Then Remove It
            claCheckers.Checkers.Remove(ToBeRemoved);
            //This Works Because If The Checker Has Already Been Removed
            //The Program Tries To Find The Checker We Pass In
            //If It Does Not, It Simply Ignores It Because It Does Not Exist.

            //This Runs Through All Data Stored In Classes And Displays It
            ResetGrid();

            //Set Possible Movement Options (Yellow)
            foreach (var Point in PosMoves)
            {
                //Foreach Possible Movement, Display As Color Yellow
                Color(Point.X, Point.Y, Colors.Yellow);
            }

            //Draws The Grid For Us.
            DrawGrid();

            CheckWin();
            #endregion
        }
    }
}
